// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

using System;
using Microsoft.Data.Caching;

namespace PlanMyNight.Models.Caching
{
    public class VelocityProvider : ICachingProvider
    {
        public bool HasKey(string container, string key)
        {
            using (var cacheFactory = new DataCacheFactory())
            {
                try
                {
                    var cacheContainer = cacheFactory.GetCache(container);
                    return cacheContainer.Get(key) != null;
                }
                catch (DataCacheException ex)
                {
                    if (ex.ErrorCode == DataCacheErrorCode.RetryLater)
                    {
                        // temporal failure, ignore and continue
                        return false;
                    }

                    throw;
                }
            }
        }

        public object Get(string container, string key)
        {
            using (var cacheFactory = new DataCacheFactory())
            {
                try
                {
                    var cacheContainer = cacheFactory.GetCache(container);
                    return cacheContainer.Get(key);
                }
                catch (DataCacheException ex)
                {
                    if (ex.ErrorCode == DataCacheErrorCode.RetryLater)
                    {
                        // temporal failure, ignore and continue
                        return null;
                    }

                    throw;
                }
            }
        }

        public void Add(string container, string key, object value, TimeSpan timeout)
        {
            if (value == null) return;

            using (var cacheFactory = new DataCacheFactory())
            {
                try
                {
                    var cacheContainer = cacheFactory.GetCache(container);
                    cacheContainer.Put(key, value, timeout);
                }
                catch (DataCacheException ex)
                {
                    if (ex.ErrorCode == DataCacheErrorCode.RetryLater)
                    {
                        // temporal failure, ignore and continue
                        return;
                    }

                    throw;
                }
            }
        }

        public void Remove(string container, string key)
        {
            using (var cacheFactory = new DataCacheFactory())
            {
                try
                {
                    var cacheContainer = cacheFactory.GetCache(container);
                    cacheContainer.Remove(key);
                }
                catch (DataCacheException ex)
                {
                    if (ex.ErrorCode == DataCacheErrorCode.KeyDoesNotExist)
                    {
                        return;
                    }

                    throw;
                }
            }
        }
    }
}